/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest.cloud.aws;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;

/**
 * This class provides functions to help creating key stores and trust stores for initializing the SSL context. It uses
 * the properties: <br>
 * - {@value #AWS_CERTIFICATE_LIST}: a list of certificate to load seperated by coma.<br>
 * - {@value #AWS_PRIVATE_KEY_FILE_PASSWORD}: password of the keystore used.<br>
 * - {@value #AWS_PRIVATE_KEY_FILE_PATH}: path to the device's private key file.<br>
 * - {@value #AWS_CERTIFICATE_FILE_PATH}: path to the device's certificate file.<br>
 */
public class SslContextHelper {

	private static final String CERTIFICATE = "certificate."; //$NON-NLS-1$

	private static final String KEY = "key."; //$NON-NLS-1$

	private static final String PRIVATE = "private."; //$NON-NLS-1$

	private static final String PATH = "path"; //$NON-NLS-1$

	private static final String DEVICE = "device."; //$NON-NLS-1$

	private static final String AWS = "aws."; //$NON-NLS-1$

	private static final String PRIVATE_KEY_NAME = "privatekey"; //$NON-NLS-1$

	/**
	 * Property to get the list of certificate to load. Expects a list of certificate to load separated by comma. (e.g.
	 * {@value #AWS_CERTIFICATE_LIST}=path/to/root.crt,path/to/leaf.crt)
	 */
	public static final String AWS_CERTIFICATE_LIST = AWS + CERTIFICATE + "list"; //$NON-NLS-1$

	/**
	 * Property to get the password of device's private key. (e.g. {@value #AWS_PRIVATE_KEY_FILE_PASSWORD}=awsdemo)
	 */
	public static final String AWS_PRIVATE_KEY_FILE_PASSWORD = AWS + DEVICE + PRIVATE + KEY + "password"; //$NON-NLS-1$

	/**
	 * Property to get the path to the device's private key. (e.g.
	 * {@value #AWS_PRIVATE_KEY_FILE_PATH}=/path/to/private.key.der)
	 */
	public static final String AWS_PRIVATE_KEY_FILE_PATH = AWS + DEVICE + PRIVATE + KEY + PATH;

	/**
	 * Property to get the path to the device's certificate. (e.g.
	 * {@value #AWS_CERTIFICATE_FILE_PATH}=/path/to/certificate.pem.crt.txt).
	 */
	public static final String AWS_CERTIFICATE_FILE_PATH = AWS + DEVICE + CERTIFICATE + PATH;

	/**
	 * TLSv1.2.
	 */
	public static final String TLS_V_1_2 = "TLSv1.2"; //$NON-NLS-1$
	/**
	 * X509.
	 */
	public static final String X509 = "X509"; //$NON-NLS-1$

	private SslContextHelper() {
		// Forbid instantiation.
	}

	/**
	 * Gets the SSL Context for AWS. It expects the properties: <br>
	 * - {@value #AWS_CERTIFICATE_LIST}: a list of certificate to load separated by comma.<br>
	 * - {@value #AWS_PRIVATE_KEY_FILE_PASSWORD}: password of the keystore used.<br>
	 * - {@value #AWS_PRIVATE_KEY_FILE_PATH}: path to the device's private key file.<br>
	 * - {@value #AWS_CERTIFICATE_FILE_PATH}: path to the device's certificate file.<br>
	 *
	 * @param properties
	 *            set of properties to use.
	 * @param certificateLoader
	 *            the certificate loader to use.
	 *
	 * @return the SSL Context.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws IOException
	 *             if IOException occurred.
	 * @throws CertificateException
	 *             if CertificateException occurred.
	 * @throws KeyStoreException
	 *             if KeyStoreException occurred.
	 * @throws UnrecoverableKeyException
	 *             if UnrecoverableKeyException occurred.
	 * @throws KeyManagementException
	 *             if NoSuchAlgorithmException occurred.
	 * @see #getDefaultKeyManagers(CertificateLoader, Properties)
	 * @see #getTrustManagers(CertificateLoader, Properties)
	 */
	public static SSLContext getSslContext(Properties properties, CertificateLoader certificateLoader)
			throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException,
			UnrecoverableKeyException, KeyManagementException {
		KeyManager[] keyManagers = getDefaultKeyManagers(certificateLoader, properties);
		TrustManager[] trustManagers = getTrustManagers(certificateLoader, properties);
		return getSslContext(keyManagers, trustManagers);
	}

	/**
	 * Gets the SSL Context for AWS.
	 *
	 * @param keyManagers
	 *            the key managers.
	 * @param trustManagers
	 *            the trust managers.
	 *
	 * @return the SSL Context.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws KeyManagementException
	 *             if NoSuchAlgorithmException occurred.
	 */
	public static SSLContext getSslContext(KeyManager[] keyManagers, TrustManager[] trustManagers)
			throws NoSuchAlgorithmException, KeyManagementException {
		SSLContext context = SSLContext.getInstance(TLS_V_1_2);
		context.init(keyManagers, trustManagers, null);
		return context;
	}

	/**
	 * Gets the trust managers, generated based on the property : <br>
	 * - {@value #AWS_CERTIFICATE_LIST}: a list of certificate to load separated by comma. (e.g.
	 * {@value #AWS_CERTIFICATE_LIST}=path/to/root.crt,path/to/leaf.crt)
	 *
	 * @param certificateLoader
	 *            the loader to use for each certificate file.
	 * @param properties
	 *            the properties set to use.
	 *
	 * @return an array of trust managers.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws IOException
	 *             if IOException occurred.
	 * @throws CertificateException
	 *             if CertificateException occurred.
	 * @throws KeyStoreException
	 *             if KeyStoreException occurred.
	 */
	public static TrustManager[] getTrustManagers(CertificateLoader certificateLoader, Properties properties)
			throws NoSuchAlgorithmException, IOException, KeyStoreException, CertificateException {
		// Populate a new keystore with the certificates listed in the client
		// certificates file
		KeyStore store = getNewKeyStore();
		for (String certificateFilename : loadCertificatesFilenames(getProperty(properties, AWS_CERTIFICATE_LIST))) {
			store.setCertificateEntry(certificateFilename,
					generateCertificateFromFile(certificateLoader, certificateFilename));
		}

		// Initialize the trust store and return the trust managers
		TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(X509);
		trustManagerFactory.init(store);
		return trustManagerFactory.getTrustManagers();
	}

	/**
	 * Gets the key managers, generated based on properties : <br>
	 * - {@value #AWS_PRIVATE_KEY_FILE_PASSWORD}: password of the password of device's private key.<br>
	 * - {@value #AWS_PRIVATE_KEY_FILE_PATH}: path to the device's private key file.<br>
	 * - {@value #AWS_CERTIFICATE_FILE_PATH}: path to the device's certificate file.<br>
	 *
	 * @param certificateLoader
	 *            the loader to use for each certificate file.
	 * @param properties
	 *            the properties set to use.
	 * @return an array of key managers.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws IOException
	 *             if IOException occurred.
	 * @throws CertificateException
	 *             if CertificateException occurred.
	 * @throws KeyStoreException
	 *             if KeyStoreException occurred.
	 * @throws UnrecoverableKeyException
	 *             if UnrecoverableKeyException occurred.
	 */
	public static KeyManager[] getDefaultKeyManagers(CertificateLoader certificateLoader, Properties properties)
			throws CertificateException, IOException, NoSuchAlgorithmException, KeyStoreException,
			UnrecoverableKeyException {
		String deviceCertificateFileName = getProperty(properties, AWS_CERTIFICATE_FILE_PATH);
		String devicePrivateKeyFileName = getProperty(properties, AWS_PRIVATE_KEY_FILE_PATH);
		String keyStorePassword = getProperty(properties, AWS_PRIVATE_KEY_FILE_PASSWORD);

		return getKeyManagers(certificateLoader, deviceCertificateFileName, devicePrivateKeyFileName, keyStorePassword);
	}

	/**
	 * Gets the key managers.
	 *
	 * @param certificateLoader
	 *            the loader to use for each certificate file.
	 *
	 * @param certificateFileName
	 *            the certificate file name.
	 * @param keyFileName
	 *            the file name of the private key associated to the certificate.
	 * @param keyStorePassword
	 *            the password of the key store.
	 *
	 * @return an array of key managers.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws IOException
	 *             if IOException occurred.
	 * @throws CertificateException
	 *             if CertificateException occurred.
	 * @throws KeyStoreException
	 *             if KeyStoreException occurred.
	 * @throws UnrecoverableKeyException
	 *             if UnrecoverableKeyException occurred.
	 */
	public static KeyManager[] getKeyManagers(CertificateLoader certificateLoader, String certificateFileName,
			String keyFileName, String keyStorePassword) throws CertificateException, IOException,
			NoSuchAlgorithmException, KeyStoreException, UnrecoverableKeyException {
		// Generate the certificate and its key
		Certificate clientCert = generateCertificateFromFile(certificateLoader, certificateFileName);
		byte[] key = loadResource(certificateLoader, keyFileName);

		return getKeyManagers(keyStorePassword, clientCert, key);
	}

	/**
	 * Gets the key managers.
	 *
	 * @param keyStorePassword
	 *            the password of the key store.
	 * @param clientCert
	 *            the certificate of the client.
	 * @param key
	 *            the private key.
	 *
	 * @return an array of key managers.
	 * @throws NoSuchAlgorithmException
	 *             if NoSuchAlgorithmException occurred.
	 * @throws IOException
	 *             if IOException occurred.
	 * @throws CertificateException
	 *             if CertificateException occurred.
	 * @throws KeyStoreException
	 *             if KeyStoreException occurred.
	 * @throws UnrecoverableKeyException
	 *             if UnrecoverableKeyException occurred.
	 */
	public static KeyManager[] getKeyManagers(String keyStorePassword, Certificate clientCert, byte[] key)
			throws NoSuchAlgorithmException, CertificateException, IOException, KeyStoreException,
			UnrecoverableKeyException {
		// Populate a new key store
		KeyStore store = getNewKeyStore();
		store.setKeyEntry(PRIVATE_KEY_NAME, key, new Certificate[] { clientCert });

		// Initialize the key managers and return them
		KeyManagerFactory km = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
		km.init(store, keyStorePassword.toCharArray());
		return km.getKeyManagers();
	}

	/**
	 * Gets a property from a property file, if not found tries the {@link System#getProperty(String)}
	 *
	 * @param properties
	 *            the properties.
	 * @param key
	 *            the key to look for.
	 * @return the property.
	 * @throws IOException
	 *             if the property is not found.
	 */
	public static String getProperty(Properties properties, String key) throws IOException {
		String property = properties.getProperty(key);
		if (property == null) {
			property = System.getProperty(key);
		}
		if (property == null) {
			throw new IOException("Property " + key + " not found"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return property;
	}

	private static Certificate generateCertificateFromFile(CertificateLoader certificateLoader,
			String certificateFilename) throws IOException, CertificateException {
		Certificate cert = null;
		try (InputStream in = certificateLoader.loadCertificate(certificateFilename)) {
			if (in == null) {
				throw new IOException(certificateFilename);
			}

			// Generate the server certificates
			cert = CertificateFactory.getInstance(X509).generateCertificate(in);
		}

		return cert;
	}

	/**
	 * Create a new Key store.
	 *
	 * @return a new key store.
	 * @throws NoSuchAlgorithmException
	 *             if the algorithm used to checkthe integrity of the keystore cannot be found
	 * @throws CertificateException
	 *             if any of the certificates in thekeystore could not be loaded
	 * @throws IOException
	 *             if there is an I/O or format problem with thekeystore data, if a password is required but not
	 *             given,or if the given password was incorrect. If the error is due to awrong password, the cause of
	 *             the IOException should be an UnrecoverableKeyException
	 * @throws KeyStoreException
	 *             if no Provider supports aKeyStoreSpi implementation for thespecified type.
	 */
	public static KeyStore getNewKeyStore()
			throws NoSuchAlgorithmException, CertificateException, IOException, KeyStoreException {
		KeyStore store = KeyStore.getInstance(KeyStore.getDefaultType());
		store.load(null, null);
		return store;
	}

	/**
	 * Loads and returns the filenames of the certificates.
	 *
	 * @param fileName
	 *            the file listing the certificates.
	 *
	 * @throws IOException
	 *             on IO Error.
	 */
	private static Set<String> loadCertificatesFilenames(String fileName) {
		int current = 0;
		Set<String> certificates = new HashSet<>();
		while (current != -1) {
			int indexOf = fileName.indexOf(',', current);
			if (indexOf < 0) {
				addfile(fileName.substring(current), certificates);
				break;
			}
			addfile(fileName.substring(current, indexOf), certificates);
			current = indexOf + 1;
		}

		return certificates;
	}

	private static void addfile(String fileName, Set<String> certificates) {
		fileName = fileName.trim();
		if (fileName.length() > 0) {
			certificates.add(fileName);
		}
	}

	/**
	 * Loads the content of a resource into a byte array.
	 * <p>
	 * This method uses {@link Class#getResourceAsStream(String)} to load the resource.
	 *
	 * @param resourceName
	 *            name of the resource
	 *
	 * @return an array of bytes filled-in with the content of the resource.
	 *
	 * @throws IOException
	 *             if an I/O error occurs during the stream reading.
	 *
	 * @see Class#getResourceAsStream(String)
	 */
	private static byte[] loadResource(CertificateLoader certificateLoader, String resourceName) throws IOException {
		try (InputStream stream = certificateLoader.loadCertificate(resourceName)) {
			if (stream == null) {
				return null;
			}

			DataInputStream dataInputStream = new DataInputStream(stream);
			byte[] data = new byte[stream.available()];

			dataInputStream.readFully(data);

			return data;
		}
	}

}
