/*
 * Java
 *
 * 	Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 *  For demonstration purpose only.
 *  MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest.endpoint;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Map;
import java.util.Properties;

import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;

import com.microej.demo.thermorest.HeadlessThermostat;
import com.microej.demo.thermorest.Messages;
import com.microej.demo.thermorest.cloud.CloudConnector;
import com.microej.demo.thermorest.cloud.aws.CertificateLoader;
import com.microej.demo.thermorest.cloud.aws.SslContextHelper;
import com.microej.demo.thermorest.cloud.aws.ThermostatAWS;

import ej.hoka.http.HTTPRequest;
import ej.hoka.http.HTTPResponse;
import ej.hoka.http.body.HTTPPart;
import ej.hoka.http.body.MultiPartBodyParser;
import ej.restserver.RestEndpoint;
import ej.util.message.Level;

/**
 * Endpoint to receive aws credentials..
 */
public class AWSUpload extends RestEndpoint {

	private static final int BUFFER_SIZE = 512;
	private static final String PASS = "pass"; //$NON-NLS-1$
	private static final String URI = "/awsUpload"; //$NON-NLS-1$

	private final CertificateLoader loader;
	private final Properties properties;
	private final HeadlessThermostat thermostat;

	/**
	 * Instantiates an endpoint to receive aws credentials.
	 *
	 * @param thermostat
	 *            the thermostat.
	 * @param loader
	 *            the certificate loader to use for the server certificates.
	 * @param properties
	 *            the properties to use for the server certificates list.
	 */
	public AWSUpload(HeadlessThermostat thermostat, CertificateLoader loader, Properties properties) {
		super(URI);
		this.thermostat = thermostat;
		this.loader = loader;
		this.properties = properties;
	}

	@Override
	public HTTPResponse post(HTTPRequest request, Map<String, String> headers, Map<String, String> parameters) {

		MultiPartBodyParser bodyParser = new MultiPartBodyParser();

		Certificate clientCertificate = null;
		byte[] key = null;
		try {
			bodyParser.parseBody(request);
			while (true) {
				try (HTTPPart partIS = bodyParser.nextPart()) {
					if (partIS == null) {
						break;
					}
					Map<String, String> parseHeaders = partIS.getHeaders();
					String name = parseHeaders.get("name"); //$NON-NLS-1$
					if (name != null) {
						if (PASS.equalsIgnoreCase(name)) {
							this.properties.put(PASS, new String(read(partIS)).trim());
						} else if ("url".equalsIgnoreCase(name)) { //$NON-NLS-1$
							this.properties.put(ThermostatAWS.AWS_BROKER_URL, new String(read(partIS)).trim());
						} else if ("port".equalsIgnoreCase(name)) { //$NON-NLS-1$
							this.properties.put(ThermostatAWS.AWS_BROKER_PORT, new String(read(partIS)).trim());
						} else if ("id".equalsIgnoreCase(name)) { //$NON-NLS-1$
							this.properties.put(ThermostatAWS.AWS_DEVICE_ID, new String(read(partIS)).trim());
						} else if ("der".equalsIgnoreCase(name)) { //$NON-NLS-1$
							key = read(partIS);
						} else if ("crt".equalsIgnoreCase(name)) { //$NON-NLS-1$
							clientCertificate = CertificateFactory.getInstance(SslContextHelper.X509)
									.generateCertificate(partIS);
						}
					}

				}
			}

			createAWSCloudConnector(clientCertificate, key);
			return HTTPResponse.RESPONSE_OK;
		} catch (IndexOutOfBoundsException | IOException | CertificateException | NoSuchAlgorithmException
				| KeyStoreException | KeyManagementException | UnrecoverableKeyException e) {
			Messages.LOGGER.log(Level.INFO, Messages.CATEGORY, Messages.CLOUD_ERROR, e);
		}
		return HTTPResponse.RESPONSE_BAD_REQUEST;
	}

	private void createAWSCloudConnector(Certificate clientCertificate, byte[] key) throws UnrecoverableKeyException,
			NoSuchAlgorithmException, CertificateException, KeyStoreException, IOException, KeyManagementException {
		KeyManager[] keyManagers = createKeymanager(clientCertificate, key, this.properties.getProperty(PASS));
		if (keyManagers != null) {
			TrustManager[] trustManagers = SslContextHelper.getTrustManagers(this.loader, this.properties);
			SSLContext sslContext = SslContextHelper.getSslContext(keyManagers, trustManagers);
			ThermostatAWS thermostatAWS = new ThermostatAWS(this.properties, sslContext);
			for (CloudConnector cloudConnector : this.thermostat.getCloudConnectors()) {
				cloudConnector.stop();
			}
			thermostatAWS.setThermostat(this.thermostat);
			thermostatAWS.start();
		}
	}

	private KeyManager[] createKeymanager(Certificate clientCertificate, byte[] key, String pass)
			throws UnrecoverableKeyException, NoSuchAlgorithmException, CertificateException, KeyStoreException,
			IOException {
		KeyManager[] managers = null;
		if (clientCertificate != null && key != null && pass != null) {
			managers = SslContextHelper.getKeyManagers(pass, clientCertificate, key);
		}
		return managers;
	}

	private static byte[] read(InputStream stream) throws IOException {
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();

		int nRead;
		byte[] data = new byte[BUFFER_SIZE];
		while ((nRead = stream.read(data, 0, data.length)) != -1) {
			buffer.write(data, 0, nRead);
		}

		return buffer.toByteArray();
	}

}
