/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.profiling.internal.snapshot;

import java.util.List;

import com.microej.profiling.Group;
import com.microej.profiling.internal.Strings;
import com.microej.profiling.internal.snapshot.Snapshot.Type;
import com.microej.profiling.profiler.ComparableProfiler;
import com.microej.profiling.profiler.HeapProfiler;

import ej.basictool.ArrayTools;

/**
 * Gather the {@link ComparableProfiler} for a thread.
 */
public class ThreadInfo {

	private static long PROFILER_SIZE;
	private final Thread thread;
	private int level;
	private final ProfilerSnapshots[] profilers;

	static {
		// Compute the size of a ProfilerSnapshots.
		Runtime runtime = Runtime.getRuntime();
		ProfilerSnapshots[] snapshots = new ProfilerSnapshots[0];
		HeapProfiler heapProfiler = new HeapProfiler();
		snapshots = ArrayTools.add(snapshots, new ProfilerSnapshots(heapProfiler));
		System.gc();
		long memory = runtime.freeMemory();
		snapshots = ArrayTools.add(snapshots, new ProfilerSnapshots(heapProfiler));
		System.gc();
		PROFILER_SIZE = memory - runtime.freeMemory();
	}

	/**
	 * Instantiates a {@link ThreadInfo}.
	 *
	 * @param profilers
	 *            the profilers to follow.
	 * @param thread
	 *            the thread followed.
	 */
	public ThreadInfo(List<ComparableProfiler> profilers, Thread thread) {
		int size = profilers.size();
		this.profilers = new ProfilerSnapshots[size];
		for (int i = 0; i < size; i++) {
			this.profilers[i] = new ProfilerSnapshots(profilers.get(i));
			HeapProfiler.OFFSET += PROFILER_SIZE;
		}
		this.thread = thread;
		this.level = 0;
	}

	/**
	 * Takes a snapshot.
	 *
	 * @param tag
	 *            the tag.
	 * @param type
	 *            the type of snapshot.
	 * @param group
	 *            the profiler group
	 */
	public void snapshot(String tag, Type type, Group group) {
		for (ProfilerSnapshots profilerSnapshots : this.profilers) {
			profilerSnapshots.snapshot(this.level, tag, type, group);
		}
	}

	/**
	 * Gets the {@link Thread}.
	 *
	 * @return the Thread.
	 */
	public Thread getThread() {
		return this.thread;
	}

	@Override
	public String toString() {
		StringBuilder out = new StringBuilder(this.thread.getName() + Strings.COLLON + Strings.LINE_BREAK);
		for (ProfilerSnapshots profilerSnapshots : this.profilers) {
			out.append(profilerSnapshots);
		}
		return out.toString();
	}

	/**
	 * Increases the stack's level.
	 */
	public void increaseStackLevel() {
		this.level++;
	}

	/**
	 * Decreases the stack's level.
	 */
	public void decreaseStackLevel() {
		this.level--;
	}

}
