/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.wadapps.connectivity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import android.net.ConnectivityManager;
import android.net.Network;
import android.net.NetworkCapabilities;
import android.net.NetworkInfo;
import android.net.NetworkRequest;
import android.net.NetworkRequest.Builder;
import ej.bon.Timer;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.kf.Feature;
import ej.kf.Feature.State;
import ej.kf.FeatureStateListener;
import ej.kf.Kernel;
import ej.kf.Module;
import ej.net.PollerConnectivityManager;

/**
 * A connectivity manager able to support Wadapps.
 */
public final class WadappsConnectivityManager extends ConnectivityManager implements FeatureStateListener {
	private static final Map<Module, List<KernelNetworkCallback>> Callbacks = new HashMap<>();
	private ConnectivityManager manager;

	/**
	 * Instantiates a {@link WadappsConnectivityManager}.
	 */
	public WadappsConnectivityManager() {
		Kernel.addFeatureStateListener(this);
		getManager();
	}

	/**
	 * @see android.net.ConnectivityManager#getActiveNetworkInfo()
	 */
	@Override
	public NetworkInfo getActiveNetworkInfo() {
		return this.getManager().getActiveNetworkInfo();
	}

	/**
	 * @see android.net.ConnectivityManager#getActiveNetwork()
	 */
	@Override
	public Network getActiveNetwork() {
		return this.getManager().getActiveNetwork();
	}

	/**
	 * @see android.net.ConnectivityManager#getAllNetworks()
	 */
	@Override
	public Network[] getAllNetworks() {
		return this.getManager().getAllNetworks();
	}

	/**
	 * @see android.net.ConnectivityManager#getNetworkCapabilities(android.net.Network)
	 */
	@Override
	public NetworkCapabilities getNetworkCapabilities(Network network) {
		return this.getManager().getNetworkCapabilities(network);
	}

	/**
	 * @see android.net.ConnectivityManager#getNetworkInfo(android.net.Network)
	 */
	@Override
	public NetworkInfo getNetworkInfo(Network network) {
		return this.getManager().getNetworkInfo(network);
	}

	/**
	 * @see android.net.ConnectivityManager#registerDefaultNetworkCallback(android.net.ConnectivityManager.NetworkCallback)
	 */
	@Override
	public void registerDefaultNetworkCallback(NetworkCallback networkCallback) {
		synchronized (Callbacks) {
			Module contextOwner = Kernel.getContextOwner();
			Kernel.enter();
			KernelNetworkCallback callback = add(contextOwner, networkCallback);
			this.getManager().registerDefaultNetworkCallback(callback);
			Kernel.exit();
		}
	}

	/**
	 * @see android.net.ConnectivityManager#registerNetworkCallback(android.net.NetworkRequest,
	 *      android.net.ConnectivityManager.NetworkCallback)
	 */
	@Override
	public void registerNetworkCallback(NetworkRequest request, NetworkCallback networkCallback) {
		synchronized (Callbacks) {
			Module contextOwner = Kernel.getContextOwner();
			Kernel.enter();

			NetworkRequest kernelRequest = null;
			if (request != null) {
				Builder builder = new NetworkRequest.Builder();
				if (request.networkCapabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)) {
					builder.addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET);
				}
				kernelRequest = builder.build();
			}
			KernelNetworkCallback callback = add(contextOwner, networkCallback);
			this.getManager().registerNetworkCallback(kernelRequest, callback);
			Kernel.exit();
		}
	}

	/**
	 * @see android.net.ConnectivityManager#unregisterNetworkCallback(android.net.ConnectivityManager.NetworkCallback)
	 */
	@Override
	public void unregisterNetworkCallback(NetworkCallback networkCallback) {
		synchronized (Callbacks) {
			Module contextOwner = Kernel.getContextOwner();
			Kernel.enter();
			KernelNetworkCallback remove = remove(contextOwner, networkCallback);
			this.getManager().unregisterNetworkCallback(remove);
			Kernel.exit();
		}
	}

	@Override
	public void stateChanged(Feature feature, State previousState) {
		State state = feature.getState();
		if (state.equals(State.STOPPED) || state.equals(State.UNINSTALLED)) {
			// clean up network callbacks
			synchronized (Callbacks) {
				Kernel.enter();
				List<KernelNetworkCallback> list = Callbacks.remove(feature);
				if (list != null) {
					for (Iterator<KernelNetworkCallback> iterator = list.iterator(); iterator.hasNext();) {
						NetworkCallback kernelNetworkCallback = iterator.next();
						this.getManager().unregisterNetworkCallback(kernelNetworkCallback);
					}
				}
				Kernel.exit();
			}
		}
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void setActiveNetwork(Network network) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void notifyNetworkCallbacks(boolean available) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void notifyNetworkCallbacks(Network network, boolean available) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void notifyNetworkCallbacks(Network network, NetworkCapabilities capabilities) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void setAvailable(boolean available, Network network, NetworkInfo networkInfo) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void doNotifyCapabilitiesChange(Network network, NetworkCapabilities capabilities,
			NetworkCallback[] networkCallBacks) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Not supported.
	 */
	@Override
	protected void doNotifyAvailabilityChange(Network network, boolean available, NetworkCallback[] networkCallbacks) {
		// Forbid the usage, this.manager should be used
		throw new RuntimeException();
	}

	/**
	 * Gets the manager. Instantiates it if needed.
	 *
	 * @return the manager.
	 */
	private synchronized ConnectivityManager getManager() {
		if (this.manager == null) {
			this.manager = new PollerConnectivityManager(
					ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class));
		}
		return this.manager;
	}

	private KernelNetworkCallback add(Module contextOwner, NetworkCallback networkCallback) {
		List<KernelNetworkCallback> list = Callbacks.get(contextOwner);
		if (list == null) {
			list = new ArrayList<>();
			Callbacks.put(contextOwner, list);
		}
		KernelNetworkCallback kernelNetworkCallback = new KernelNetworkCallback(networkCallback);
		list.add(kernelNetworkCallback);
		return kernelNetworkCallback;
	}

	private KernelNetworkCallback remove(Module contextOwner, NetworkCallback networkCallback) {
		KernelNetworkCallback callback = null;
		List<KernelNetworkCallback> list = Callbacks.get(contextOwner);
		if (list != null) {
			for (KernelNetworkCallback kernelNetworkCallback : list) {
				if (kernelNetworkCallback.getFeatureCallback().equals(networkCallback)) {
					list.remove(networkCallback);
					callback = kernelNetworkCallback;
					break;
				}
			}
		}
		return callback;
	}

}
