/*
 * Java
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.wadapps.management.util;

import java.util.logging.Level;
import java.util.logging.Logger;

import ej.observable.Observer;
import ej.wadapps.app.Activity;
import ej.wadapps.management.ActivitiesScheduler;
import ej.wadapps.management.LaunchersList;

/**
 * Shows first registered launcher. If this launcher is removed, show the next one available.
 */
public class LaunchersManager implements Observer {

	private LaunchersList launchersList;
	private ActivitiesScheduler activitiesScheduler;
	// private Activity currentLauncher;
	private boolean started;

	/**
	 * Default constructor.
	 */
	public LaunchersManager() {
	}

	/**
	 * Sets the activities scheduler that holds the current launcher.
	 *
	 * @param activitiesScheduler
	 *            the activities scheduler to set.
	 * @throws IllegalStateException
	 *             if the launcher manager is already started.
	 */
	public synchronized void setActivitiesScheduler(ActivitiesScheduler activitiesScheduler)
			throws IllegalStateException {
		if (this.started) {
			throw new IllegalStateException();
		}
		this.activitiesScheduler = activitiesScheduler;
	}

	/**
	 * Sets the launchers list to manage.
	 * <p>
	 * Starts all launchers registered in the background list.
	 *
	 * @param launchersList
	 *            the launchers list to set.
	 */
	public synchronized void setLaunchersList(LaunchersList launchersList) {
		internalStop();
		this.launchersList = launchersList;
		internalStart();
	}

	/**
	 * Starts the background manager.
	 * <p>
	 * Starts all launchers registered in the background list.
	 */
	public synchronized void start() {
		this.started = true;
		internalStart();
	}

	/**
	 * Stops the background manager.
	 * <p>
	 * Starts all started launchers.
	 */
	public synchronized void stop() {
		this.started = false;
		internalStop();
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void internalStart() {
		LaunchersList launchersList = this.launchersList;
		if (this.started && launchersList != null) {
			launchersList.addObserver(this);
			startFirstLauncher(launchersList);
		}
	}

	private void startFirstLauncher(LaunchersList launchersList) {
		// retrieve first launcher from registered launchers
		Activity[] launchers = launchersList.getLaunchers();
		if (launchers.length >= 1) {
			Activity activity = launchers[0];
			startCurrentLauncher(activity);
		} else {
			Logger.getLogger(LaunchersManager.class.getName()).log(Level.FINE, "No registered launcher!");
		}
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void internalStop() {
		stopCurrentLauncher();
		LaunchersList launchersList = this.launchersList;
		if (launchersList != null) {
			launchersList.deleteObserver(this);
		}
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void startCurrentLauncher(Activity launcher) {
		this.activitiesScheduler.setLauncher(launcher);
		// launcher.create();
		// launcher.start();
		// this.currentLauncher = launcher;
	}

	/**
	 * Must be called in a synchronized context.
	 */
	private void stopCurrentLauncher() {
		this.activitiesScheduler.setLauncher(null);
		// if (this.currentLauncher != null) {
		// this.currentLauncher.stop();
		// this.currentLauncher.destroy();
		// this.currentLauncher = null;
		// }
	}

	@Override
	public synchronized void update() {
		Activity[] launchers = this.launchersList.getLaunchers();
		// check if current launcher is always available
		Activity currentLauncher = this.activitiesScheduler.getLauncher();
		if (currentLauncher != null && !contains(launchers, currentLauncher)) {
			stopCurrentLauncher();
		}
		if (currentLauncher == null) {
			// try to find a launcher to launch
			startFirstLauncher(this.launchersList);
		}
	}

	private boolean contains(Activity[] activities, Activity activity) {
		for (Activity candidate : activities) {
			if (candidate == activity) {
				return true;
			}
		}
		return false;
	}

}