/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver;

import java.io.IOException;

/**
 * The thermostat modes.
 */
public interface ThermostatMode {

	/** System is OFF. */
	int OFF               = 0x00;

	/** Continuous heating only. */
	int HEAT              = 0x01;

	/** Continuous cooling only. */
	int COOL              = 0x02;

	/**
	 * The system will automatically switch between heating and cooling when the temperature exceeds the HEAT and COOL
	 * set point types.
	 */
	int AUTO              = 0x03;

	/**
	 * Auxiliary/Emergency Heat. A heat pump (especially air exchange types) is not efficient when the outside
	 * temperature is below 35 degrees Fahrenheit (~0 degrees centigrade). Thus, the thermostat may be put into
	 * auxiliary heat mode simply to use a more efficient secondary heat source when there are no failures of the
	 * compressor or heat pump unit itself.
	 */
	int AUXILIARY         = 0x04;

	/**
	 * The system MUST resume to last active mode. The Thermostat Mode Report command MUST NOT advertise this Mode
	 * identifier.
	 */
	int RESUME            = 0x05;

	/** Fan only - cycle fan to circulate air. */
	int FAN               = 0x06;

	/** Cycle fan to circulate air - heating or cooling will be activated according to the FURNACE set point. */
	int FURNACE           = 0x07;

	/**
	 * Dehumidification - The system will cycle cooling in relation to the room and the DRY set point temperature in
	 * order to remove moisture from ambient.
	 */
	int DRY               = 0x08;

	/** Humidification - Moist Air, heating or cooling will be activated according to the MOIST set point. */
	int MOIST             = 0x09;

	/** Auto Changeover - heating or cooling will be activated according to the AUTO CHANGEOVER set point. */
	int AUTO_CHANGEOVER   = 0x0A;

	/**
	 * Energy Saving Heating (usually lower than normal set point) - heating will be activated according to the ENERGY
	 * HEAT set point.
	 */
	int ENERGY_HEAT       = 0x0B;

	/**
	 * Energy Saving Cooling (usually higher than normal set point) - cooling will be activated according to the ENERGY
	 * COOL set point.
	 */
	int ENERGY_COOL       = 0x0C;

	/**
	 * Away mode, e.g. preventing water from freezing in forced water systems - heating or cooling will be activated
	 * when temperature exceeds the AWAY HEAT and/or AWAY COOL set points.
	 */
	int AWAY              = 0x0D;

	/**
	 * SPEED UP / FULL POWER heating or cooling mode will be activated when temperature exceeds FULL POWER set point.
	 */
	int FULL_POWER        = 0x0E;

	/**
	 * Gets the name of the mode.
	 *
	 * @return the type.
	 */
	int getType();

	/**
	 * Sets the value.
	 *
	 * @param value
	 *            the value.
	 * @throws IOException
	 *             if there is a communication error with the device.
	 */
	void setValue(double value) throws IOException;

	/**
	 * Gets the last known value.
	 *
	 * @return the last known value, or {@link MultilevelValue#UNKNOWN} if unknown.
	 */
	double getLastKnownValue();

	/**
	 * Requests the update of the value.
	 *
	 * @throws IOException
	 *             if there is a communication error with the device
	 */
	void requestUpdate() throws IOException;

}
