/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.data;

import java.io.IOException;

/**
 * This abstract class provides a default implementation for the {@link DataReader} interface with the following features:
 * <ul>
 * <li>Implements structured data checks mandatory by the {@link DataReader} interface (an {@link IllegalStateException} thrown on error).</li>
 * <li>Defines encoding methods with suffix <code>0</code> (e.g <code>readBoolean</code> calls <code>readBoolean0</code>). Methods can throw any kind of exceptions that will be wrapped into an {@link IOException}.</li>
 * <li>An additional {@link #end()} method is called when the data is terminated.</li>
 * </ul>
 */
public abstract class AbstractDataReader implements DataReader{

	DataStreamState state;

	public AbstractDataReader() {
		state = new DataStreamState();
	}

	@Override
	public final boolean hasMoreElements() throws IOException{
		try{
			return hasMoreElements0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
	}

	@Override
	public final boolean readBooleanValue()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_BT);
		boolean value;
		try{
			value = readBooleanValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
		return value;
	}


	@Override
	public final int readIntValue()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_BT);
		int value;
		try{
			value = readIntValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
		return value;
	}

	@Override
	public final long readLongValue()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_BT);
		long value;
		try{
			value = readLongValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
		return value;
	}

	@Override
	public final String readStringValue()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_BT);
		String value;
		try{
			value = readStringValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
		return value;
	}

	@Override
	public final double readDoubleValue()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_BT);
		double value;
		try{
			value = readDoubleValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
		return value;
	}

	@Override
	public final String readStringKey()  throws IOException{
		state.checkAndUpdateState(DataStreamState.KEY_STRING);
		try{
			return readStringKey0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
	}

	@Override
	public final int readIntKey()  throws IOException{
		state.checkAndUpdateState(DataStreamState.KEY_INT);
		try{
			return readIntKey0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
	}

	@Override
	public final void readArrayValueStart()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_ARRAY);
		try{
			readArrayValueStart0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
	}

	@Override
	public final void readArrayValueEnd()  throws IOException{
		state.checkAndUpdateStateEndCollection(DataStreamState.VALUE_ARRAY);
		try{
			readArrayValueEnd0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
	}

	@Override
	public final void readMapValueStart()  throws IOException{
		state.checkAndUpdateState(DataStreamState.VALUE_MAP);
		try{
			readMapValueStart0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
	}

	@Override
	public final void readMapValueEnd()  throws IOException{
		state.checkAndUpdateStateEndCollection(DataStreamState.VALUE_MAP);
		try{
			readMapValueEnd0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
	}

	@Override
	public final void skipValue()  throws IOException{
		state.checkSkipValue();
		try{
			skipValue0();
		}
		catch(Throwable e){
			throw new IOException(e);
		}
		checkEndOfData();
	}

	private void checkEndOfData() throws IOException{
		if(state.checkAndUpdateStateEndOfData()){
			end();
		}
	}

	/**
	 * This method is called when the data has been fully read, giving
	 * a "finalization" hook to the implementation.
	 * @throws IOException on I/O error.
	 */
	public void end() throws IOException{
		// Does nothing by default
	}

	protected abstract boolean hasMoreElements0() throws Throwable;
	protected abstract boolean readBooleanValue0() throws Throwable;
	protected abstract int readIntValue0() throws Throwable;
	protected abstract long readLongValue0() throws Throwable;
	protected abstract double readDoubleValue0() throws Throwable;
	protected abstract String readStringValue0() throws Throwable;
	protected abstract String readStringKey0() throws Throwable;
	protected abstract int readIntKey0() throws Throwable;
	protected abstract void readArrayValueStart0() throws Throwable;
	protected abstract void readArrayValueEnd0() throws Throwable;
	protected abstract void readMapValueStart0() throws Throwable;
	protected abstract void readMapValueEnd0() throws Throwable;
	protected abstract void skipValue0() throws Throwable;
}
