/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.data.pubsub;

import java.io.IOException;

import ej.data.DataWriter;

/**
 * An MQTT context with its own subscriptions to an MQTT broker. Can be used to publish data and subscribe to topics.
 */
public interface MqttContext extends AutoCloseable{

	/** MQTT Quality of Service level 0: data received at most once. */
	public static final int QOS_0 = 0;

	/** MQTT Quality of Service level 1: data received at least once. */
	public static final int QOS_1 = 1;

	/** MQTT Quality of Service level 2: data received exactly once. */
	public static final int QOS_2 = 2;

	/**
	 * Requests a new publication to the given topic and returns its associated data writer.
	 * The data will be automatically published once the caller has fully written a value in the data writer.<p>
	 * This method behaves exactly as if it simply performs the call <code>newPublication(topic, {@link #QOS_0}, false)</code>.
	 * 
	 * @param topic topic to deliver a message to.
	 * 
	 * @return the dedicated {@link DataWriter}.
	 * 
	 * @throws IOException if nothing can be published on the {@link MqttContext}.
	 * @throws NullPointerException if the given topic is <code>null</code>.
	 */
	public DataWriter newPublication(String topic) throws IOException;

	/**
	 * Requests a new publication to the given topic and returns its associated data writer.
	 * The data will be automatically published once the caller has fully written a value in the data writer.
	 * 
	 * @param topic topic to deliver a message to.
	 * @param qos MQTT quality of service ({@link #QOS_0}, {@link #QOS_1} or {@link #QOS_2}).
	 * @param retain whether or not the message should be retained by the server.
	 * 
	 * @return the dedicated {@link DataWriter}.
	 * 
	 * @throws IOException if nothing can be published on the {@link MqttContext}.
	 * @throws NullPointerException if the given topic is <code>null</code>.
	 * @throws IllegalArgumentException if the QOS is not one of ({@link #QOS_0}, {@link #QOS_1} or {@link #QOS_2}).
	 */
	public DataWriter newPublication(String topic, int qos, boolean retain) throws IOException;

	/**
	 * Sets the listener that is called when a message is received.<p>
	 * By default there is no listener defined for this {@link MqttContext}.
	 * 
	 * @param listener the listener. <code>null</code> to disable notification.
	 */
	public void setListener(MqttListener listener);

	/**
	 * Subscribes to a topic.
	 * The registered {@link MqttListener} is called when a message is received.<p>
	 * This method behaves exactly as if it simply performs the call <code>subscribe(topicFilter, {@link #QOS_0})</code>.
	 * 
	 * @param topicFilter the topic to subscribe to.
	 * 
	 * @throws IOException if subscription fails.
	 * @throws NullPointerException if the given topic filter is <code>null</code>.
	 */
	public void subscribe(String topicFilter) throws IOException;

	/**
	 * Subscribes to a topic at the given QOS.
	 * The registered {@link MqttListener} is called when a message is received.<p>
	 * The given topic filter cannot use wildcards.
	 * 
	 * @param topicFilter the topic to subscribe to.
	 * @param qos Quality of Service level at which the client wants to receive the messages ({@link #QOS_0}, {@link #QOS_1} or {@link #QOS_2}).
	 * 
	 * @throws IOException if subscription fails.
	 * @throws NullPointerException if the given topic filter is <code>null</code>.
	 * @throws IllegalArgumentException if the QOS is not one of ({@link #QOS_0}, {@link #QOS_1} or {@link #QOS_2}).
	 */
	public void subscribe(String topicFilter, int qos) throws IOException;

	/**
	 * Requests the server to unsubscribe the client from a topic.
	 * 
	 * @param topicFilter the topic to unsubscribe from. It must match a topicFilter specified on the subscribe.
	 * 
	 * @throws IOException if unregistering the subscription fails.
	 * @throws NullPointerException if the given topic filter is <code>null</code>.
	 */
	public void unsubscribe(String topicFilter) throws IOException;

	/**
	 * Returns the topic prefix to use to publish data shared with other accounts.<p>
	 * The returned {@link String} ends with a <code>'/'</code> if it is not empty.
	 * Here is an example of usage:
	 * <pre>
	 * DataWriter writer = mqttContext.newPublication(mqttContext.getSharedTopicPrefix()+"temperature");
	 * </pre>
	 * @return an MQTT topic prefix.
	 */
	public String getSharedTopicPrefix();

	/**
	 * Returns the topic prefix to use to publish data sandboxed within the current account.<p>
	 *  The returned {@link String} ends with a <code>'/'</code> if it is not empty.
	 * Here is an example of usage:
	 * <pre>
	 * DataWriter writer = mqttContext.newPublication(mqttContext.getLocalTopicPrefix()+"temperature");
	 * </pre>
	 * @return an MQTT topic prefix.
	 */
	public String getLocalTopicPrefix();

	/**
	 * Closes this {@link MqttContext}, relinquishing any underlying resources.<p>
	 * When an {@link MqttContext} has been closed, access to any of its methods that involve an I/O operation will cause an IOException to be thrown.
	 * Closing an already closed {@link MqttContext} has no effect.
	 * {@link MqttListener} registered for this {@link MqttContext} will not be called anymore.
	 * 
	 * @throws IOException If an I/O error occurs.
	 */
	@Override
	public void close() throws IOException;

}