/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.data.pubsub.paho;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.paho.client.mqttv3.IMqttAsyncClient;
import org.eclipse.paho.client.mqttv3.MqttException;

import ej.data.pubsub.MqttAgent;
import ej.data.pubsub.MqttContext;

/**
 * Class that manages all the {@link PahoMqttContext}.
 */
public class PahoMqttAgent implements MqttAgent {

	/** Name of the provider */
	private final String id;
	/** The MQTT Client used to publish/subscribe to the topics */
	protected IMqttAsyncClient mqttClient;
	/** Factory and configuration provider. */
	protected final MqttAgentConfiguration agentConfiguration;

	/** Gives for each subscribed topic, the set of {@link PahoMqttContext} that have subscribed to it. */
	protected Map<String, Set<PahoMqttContext>> subscribedTopics;

	/**
	 * Creates a new {@link MqttAgent} that uses Paho.
	 * @param id value returned by {@link #getId()}.
	 * @param mqttClient the Paho MQTT Client to use for this agent.
	 * @param agentConfiguration configuration for this agent.
	 */
	public PahoMqttAgent(String id, IMqttAsyncClient mqttClient, MqttAgentConfiguration agentConfiguration){
		this.id = id;
		this.mqttClient = mqttClient;
		this.agentConfiguration = agentConfiguration;
		this.subscribedTopics = new HashMap<>();
		mqttClient.setCallback(new PahoMqttAgentCallback(this));
	}

	@Override
	public MqttContext newMqttContext() {
		return new PahoMqttContext(this);
	}

	@Override
	public String getId() {
		return id;
	}

	/**
	 * Subscribe the given connection to the given topic.
	 * 
	 * @param connection the {@link PahoMqttContext} that subscribes.
	 * @param topicFilter the topic filter to subscribe to.
	 * @param qos the QOS of the subscription.
	 * 
	 * @throws IOException on I/O Error.
	 */
	synchronized public void subscribe(PahoMqttContext connection, String topicFilter, int qos) throws IOException {
		// Check if there is already a subscription for this topic.
		Set<PahoMqttContext> connectionsForThisTopic = subscribedTopics.get(topicFilter);
		if(connectionsForThisTopic == null){
			// First subscription for this topic. Create a new Set of connection for the topic.
			connectionsForThisTopic = new HashSet<>();
			subscribedTopics.put(topicFilter, connectionsForThisTopic);
		}
		connectionsForThisTopic.add(connection);
		try {
			// FIXME: if another connection subscribes to this topic with a different QOS, then this
			// new subscription will override the previous one with its own QOS.
			mqttClient.subscribe(topicFilter, qos);
		} catch (MqttException e) {
			throw new IOException(e);
		}
	}

	/**
	 * Remove subscription for the given connection.
	 * 
	 * @param connection the {@link PahoMqttContext} that unsubscribes.
	 * @param topicFilter the topic filter to unsubscribe to.
	 * 
	 * @throws IOException on I/O Error.
	 */
	synchronized public void unsubscribe(PahoMqttContext connection, String topicFilter) throws IOException {
		// Check if there is a subscription for this topic.
		Set<PahoMqttContext> connectionsForThisTopic = subscribedTopics.get(topicFilter);
		if(connectionsForThisTopic != null){
			// A subscription exists for this topic filter.
			connectionsForThisTopic.remove(connection);
			if(connectionsForThisTopic.isEmpty()){
				// No more subscription for this topic: unsubscribe in Paho
				try {
					mqttClient.unsubscribe(topicFilter);
					// Cleanup hashmap.
					subscribedTopics.remove(connectionsForThisTopic);
				} catch (MqttException e) {
					throw new IOException(e);
				}
			}
		}
		// else: No subscription for this topic. Nothing to do.
	}

	/**
	 * Free any data related to the given connection
	 * @param connection the connection to cleanup.
	 */
	synchronized public void cleanupConnection(PahoMqttContext connection) {
		// Remove all the entries for the given connection in subcribedTopics.

		// For each topic filter, check if the connection is subscribed to the topic filter.
		for( Entry<String, Set<PahoMqttContext>> topicFilterEntry : subscribedTopics.entrySet()){
			Set<PahoMqttContext> connections = topicFilterEntry.getValue();
			if(connections != null && connections.contains(connection)){
				// Connection subscribe to the topic filer.
				connections.remove(connection);
				if(connections.isEmpty()){
					// No more subscription for the topic filter: unsubscribe in Paho
					try {
						mqttClient.unsubscribe(topicFilterEntry.getKey());
					} catch (MqttException e) {
						// nothing to do, continue on next.
					}
				}
			}
		}

	}

	/**
	 * Returns the {@link MqttContext} that are subscribed to the given topic.<p>
	 * @param topic the MQTT topic.
	 * @return a {@link Set} of {@link PahoMqttContext}.
	 */
	synchronized public Set<PahoMqttContext> getMqttContextSubscribedTo(String topic){
		// WARNING: the returned Set must not be used internally. Do a copy before returning it.
		Set<PahoMqttContext> contexts = subscribedTopics.get(topic);
		if(contexts != null){
			// Copy the set
			contexts = new HashSet<PahoMqttContext>(contexts);
		}
		else {
			contexts = new HashSet<PahoMqttContext>();
		}
		return contexts;
	}

	/**
	 * Close this agent.
	 */
	public void close(){
		try {
			mqttClient.disconnect();
			mqttClient.disconnectForcibly();
			mqttClient.close();
		} catch (MqttException e) {
			e.printStackTrace();
		}
	}
}
