/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import java.util.StringTokenizer;

import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;

/**
 * An attribute value list selector selects by checking that a value is contained in an attribute.
 * <p>
 * The attribute value is considered as a white space-separated list of words. Checks that one of those is exactly the
 * given value.
 * <p>
 * Equivalent to <code>[attribute~=value]</code> selector in CSS. Its specificity is (0,0,1,0).
 *
 * @see Element#getAttribute(String)
 * @see SpecificityHelper
 */
public class AttributeValueListSelector implements Selector {

	private static final String SPACE = " "; //$NON-NLS-1$
	private final String attribute;
	private final String value;

	/**
	 * Creates an attribute value list selector.
	 *
	 * @param attribute
	 *            the attribute to check.
	 * @param value
	 *            the value to check.
	 */
	public AttributeValueListSelector(String attribute, String value) {
		this.attribute = attribute;
		this.value = value;
	}

	@Override
	public boolean fit(Element element) {
		String value = element.getAttribute(this.attribute);
		if (value != null) {
			StringTokenizer tokenizer = new StringTokenizer(value, SPACE);
			while (tokenizer.hasMoreTokens()) {
				String nextToken = tokenizer.nextToken();
				// Test in this order to avoid checking null value (already checked by equals).
				if (this.value.equals(nextToken)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(0, 0, 1, 0);
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof AttributeValueListSelector) {
			AttributeValueListSelector other = (AttributeValueListSelector) obj;
			return this.attribute.equals(other.attribute) && this.value.equals(other.value);
		}
		return false;
	}

	@Override
	public int hashCode() {
		return this.attribute.hashCode() * 31 + this.value.hashCode();
	}

}
