/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;

import ej.annotation.NonNull;
import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;
import ej.style.Stylesheet;
import ej.style.util.StyleHelper;

/**
 * An instance selector selects by checking the element id (Java instance).
 * <p>
 * Use it sparingly. Prefer using wider scope selectors (such as type selector or class selector) that are more in line
 * with CSS philosophy.
 * <p>
 * Equivalent to a style defined in a HTML tag (no CSS selector). Its specificity is (1,0,0,0).
 * <p>
 * When set on the global stylesheet a rule with an instance selector is automatically removed from the stylesheet rules
 * after garbage collection of the referenced element.
 * <p>
 * When set on another stylesheet or in a combinator the rule remains applicable for the lifetime of the stylesheet or
 * combinator.
 *
 * @see SpecificityHelper
 * @see StyleHelper#getStylesheet()
 * @see Stylesheet#removeRule(Selector)
 */
public class InstanceSelector implements Selector {

	private static final ReferenceQueue<Element> ELEMENT_REFERENCE_QUEUE = new ReferenceQueue<>();
	private static final Map<Reference<Element>, InstanceSelector> INSTANCE_SELECTORS = new HashMap<>(0);

	private final Reference<Element> elementReference;
	private final int hashcode;

	/**
	 * Creates an instance selector.
	 *
	 * @param element
	 *            the element instance to check.
	 */
	public InstanceSelector(Element element) {
		clean();
		this.elementReference = new WeakReference<Element>(element, ELEMENT_REFERENCE_QUEUE);
		INSTANCE_SELECTORS.put(this.elementReference, this);
		this.hashcode = element.hashCode();
	}

	private void clean() {
		do {
			Reference<? extends Element> poll = ELEMENT_REFERENCE_QUEUE.poll();
			if (poll == null) {
				break;
			}
			@SuppressWarnings("null")
			@NonNull
			InstanceSelector instanceSelector = INSTANCE_SELECTORS.get(poll);
			StyleHelper.getStylesheet().removeRule(instanceSelector);
		} while (true);
	}

	@Override
	public boolean fit(Element element) {
		return element.equals(this.elementReference.get());
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(1, 0, 0, 0);
	}

	@Override
	public int hashCode() {
		return this.hashcode;
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof InstanceSelector) {
			return ((InstanceSelector) obj).elementReference == this.elementReference
					|| ((InstanceSelector) obj).elementReference.get() == this.elementReference.get();
		}
		return false;
	}

}
