/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.wadapps.admin;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * The applications manager is responsible of the applications lifecycle.
 * <p>
 * It allows:
 * <ul>
 * <li>to install applications,</li>
 * <li>retrieve applications information.</li>
 * </ul>
 */
public interface ApplicationsManager {
	/**
	 * Decodes and executes an application management command. For example, the command can be a request to install,
	 * uninstall, start, stop an application…
	 *
	 * <p>
	 * The format of the command binary form is not specified here and depend on the implementation.
	 * <p>
	 * For example, the command could be digitally signed and encoded using Distinguished Encoding Rules, a.k.a DER,
	 * (see ASN.1).
	 *
	 * @param commandInputStream
	 *            the encoded command input stream.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws ApplicationOperationException
	 *             if an error occurs while executing the command.
	 * @throws UnsupportedOperationException
	 *             if the applications manager does not support the operation.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 */
	void execute(InputStream commandInputStream)
			throws IOException, ApplicationOperationException, UnsupportedOperationException, SecurityException;

	/**
	 * Decodes and executes an application management command. For example, the command can be a request to install,
	 * uninstall, start, stop an application…
	 *
	 * <p>
	 * The format of the command binary form is not specified here and depend on the implementation.
	 * <p>
	 * For example, the command could be digitally signed and encoded using Distinguished Encoding Rules, a.k.a DER,
	 * (see ASN.1).
	 *
	 * @param commandInputStream
	 *            the encoded command input stream.
	 * @param monitor
	 *            the progress monitor to use for reporting progress to the user. It is the caller's responsibility to
	 *            call done() on the given monitor. Accepts <code>null</code>, indicating that no progress should be
	 *            reported and that the operation cannot be cancelled.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws ApplicationOperationException
	 *             if an error occurs while executing the command.
	 * @throws UnsupportedOperationException
	 *             if the applications manager does not support the operation.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 */
	void execute(InputStream commandInputStream, IProgressMonitor monitor)
			throws IOException, ApplicationOperationException, UnsupportedOperationException, SecurityException;

	/**
	 * Installs an application from the given input stream.
	 *
	 * @param inputStream
	 *            the input stream from where the application data is loaded.
	 * @param applicationIdentifier
	 *            the application identifier.
	 * @param applicationVersion
	 *            the application version.
	 * @return the newly installed application.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws ApplicationOperationException
	 *             if an error occurs during the installation or if unsigned applications are not authorized.
	 * @throws UnsupportedOperationException
	 *             if the applications manager cannot install applications.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 */
	Application install(InputStream inputStream, String applicationIdentifier, String applicationVersion)
			throws IOException, ApplicationOperationException, UnsupportedOperationException, SecurityException;

	/**
	 * Adds an application lifecycle listener.
	 * <p>
	 * It will be notified for each application event: install, start, stop, uninstall.
	 *
	 * @param applicationLifecycleListener
	 *            the listener to add.
	 * @throws NullPointerException
	 *             if the given listener is <code>null</code>.
	 */
	void addApplicationLifecycleListener(ApplicationLifecycleListener applicationLifecycleListener)
			throws NullPointerException;

	/**
	 * Removes an application lifecycle listener.
	 * <p>
	 * If the given listener is <code>null</code> or not registered, nothing is done.
	 *
	 * @param applicationLifecycleListener
	 *            the listener to remove.
	 */
	void removeApplicationLifecycleListener(ApplicationLifecycleListener applicationLifecycleListener);

	/**
	 * Gets all installed applications.
	 * <p>
	 * If there is not installed applications, this method returns an empty array.
	 *
	 * @return all installed applications.
	 */
	Application[] getInstalledApplications();

	/**
	 * Gets the context application.
	 *
	 * @return the context application.
	 */
	Application getCurrentApplication();

	/**
	 * Gets the application that owns the given object.
	 *
	 * @param object
	 *            the object.
	 * @return the application that owns this object or null if the application has been stopped
	 */
	Application getApplication(Object object);

}
