/*
 * Java
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.wadapps.management.activators;

import java.util.HashMap;
import java.util.Map;

import com.microej.wadapps.kernel.ManagementListener;
import com.microej.wadapps.kernel.SystemEventsHandler;
import com.microej.wadapps.management.util.ActivitiesSchedulerLowMemory;
import com.microej.wadapps.management.util.BackgroundsManager;
import com.microej.wadapps.management.util.LaunchersManager;

import ej.components.dependencyinjection.ServiceLoader;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.components.registry.BundleActivator;
import ej.components.registry.util.BundleRegistryHelper;
import ej.exit.ExitHandler;
import ej.wadapps.management.ActivitiesList;
import ej.wadapps.management.ActivitiesScheduler;
import ej.wadapps.management.BackgroundServicesList;
import ej.wadapps.management.LaunchersList;
import ej.wadapps.management.NotificationsManager;
import ej.wadapps.registry.SharedRegistry;
import ej.wadapps.registry.SharedRegistryFactory;

/**
 * Default Wadapps framework activator.
 * <p>
 * Creates a low memory activities scheduler, a launchers manager and a
 * backgrounds manager.
 */
public class DefaultManagementActivator implements BundleActivator {

	private final Map<Class<? extends Object>, Object> implementations = new HashMap<>();

	private ActivitiesSchedulerLowMemory activitiesScheduler;
	private BackgroundsManager backgroundsManager;
	private LaunchersManager launchersManager;

	@Override
	public void initialize() {
		this.activitiesScheduler = new ActivitiesSchedulerLowMemory();
		BundleRegistryHelper.getRegistry().register(ActivitiesScheduler.class, this.activitiesScheduler);
		this.launchersManager = new LaunchersManager();
		this.backgroundsManager = new BackgroundsManager();
	}

	@Override
	public void link() {
		ServiceLoader serviceLoader = ServiceLoaderFactory.getServiceLoader();
		ActivitiesList activitiesList = serviceLoader.getService(ActivitiesList.class);
		this.activitiesScheduler.setActivitiesList(activitiesList);
		LaunchersList launchersList = serviceLoader.getService(LaunchersList.class);
		this.launchersManager.setActivitiesScheduler(this.activitiesScheduler);
		this.launchersManager.setLaunchersList(launchersList);
		BackgroundServicesList backgroundsList = serviceLoader.getService(BackgroundServicesList.class);
		this.backgroundsManager.setBackgroundsList(backgroundsList);

		SharedRegistry sharedRegistry = SharedRegistryFactory.getSharedRegistry();
		if (sharedRegistry != null) {
			registerGlobally(ActivitiesScheduler.class, sharedRegistry, serviceLoader);
			registerGlobally(LaunchersList.class, sharedRegistry, serviceLoader);
			registerGlobally(ActivitiesList.class, sharedRegistry, serviceLoader);
			registerGlobally(BackgroundServicesList.class, sharedRegistry, serviceLoader);
			registerGlobally(NotificationsManager.class, sharedRegistry, serviceLoader);
			registerGlobally(SystemEventsHandler.class, sharedRegistry, serviceLoader);
			registerGlobally(ExitHandler.class, sharedRegistry, serviceLoader);
		}

		ManagementListener listener = serviceLoader.getService(ManagementListener.class);
		if (listener != null) {
			listener.servicesRegistered();
		}
	}

	@Override
	public void start() {
		this.launchersManager.start();
		this.backgroundsManager.start();
	}

	@Override
	public void stop() {
		this.launchersManager.stop();
		this.backgroundsManager.stop();

		SharedRegistry sharedRegistry = SharedRegistryFactory.getSharedRegistry();
		ServiceLoader serviceLoader = ServiceLoaderFactory.getServiceLoader();
		if (sharedRegistry != null) {
			unregisterGlobally(ActivitiesScheduler.class, sharedRegistry, serviceLoader);
			unregisterGlobally(LaunchersList.class, sharedRegistry, serviceLoader);
			unregisterGlobally(ActivitiesList.class, sharedRegistry, serviceLoader);
			unregisterGlobally(BackgroundServicesList.class, sharedRegistry, serviceLoader);
			unregisterGlobally(NotificationsManager.class, sharedRegistry, serviceLoader);
			unregisterGlobally(SystemEventsHandler.class, sharedRegistry, serviceLoader);
		}

		ManagementListener listener = serviceLoader.getService(ManagementListener.class);
		if (listener != null) {
			listener.servicesUnregistered();
		}
	}

	private <T> void registerGlobally(Class<T> service, SharedRegistry sharedRegistry, ServiceLoader serviceLoader) {
		T implementation = serviceLoader.getService(service);
		this.implementations.put(service, implementation);
		sharedRegistry.register(service, implementation);
	}

	private <T> void unregisterGlobally(Class<T> service, SharedRegistry sharedRegistry, ServiceLoader serviceLoader) {
		// cast ensured by construction (see registerGlobally())
		@SuppressWarnings("unchecked")
		T implementation = (T) this.implementations.get(service);
		this.implementations.remove(service);
		sharedRegistry.unregister(service, implementation);
	}

}
