/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget;

import java.util.StringTokenizer;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.microui.display.GraphicsContext;
import ej.mwt.Composite;
import ej.mwt.MWT;
import ej.mwt.Renderable;
import ej.mwt.Widget;
import ej.style.Element;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.style.outline.Outline;
import ej.style.util.StyleHelper;

/**
 * Convenient utilities for styled elements.
 */
/* package */ class StyledHelper<E extends Renderable & Element & StyledRenderable> {

	static final String[] EMPTY_STRING_ARRAY = new String[0];

	private static final String SPACE = " "; //$NON-NLS-1$

	private StyledHelper() {
	}

	/**
	 * Gets the parent element of a widget.
	 *
	 * @param widget
	 *            the widget.
	 * @return the parent of the widget or <code>null</code> if none.
	 */
	@Nullable
	static Element getParentElement(Widget widget) {
		try {
			Composite parent = widget.getParent();
			if (parent != null) {
				return (Element) parent;
			} else {
				return (Element) widget.getPanel();
			}
		} catch (ClassCastException e) {
			return null;
		}
	}

	static boolean contains(int elementX, int elementY, int elementWidth, int elementHeight,
			@Nullable Style elementStyle, int x, int y) {
		Rectangle bounds = new Rectangle(elementX, elementY, elementWidth, elementHeight);
		if (elementStyle != null) {
			Outline margin = elementStyle.getMargin();
			margin.unwrap(bounds);
		}
		int boundsX = bounds.getX();
		int boundsY = bounds.getY();
		return x >= boundsX && x <= boundsX + bounds.getWidth() && y >= boundsY && y <= boundsY + bounds.getHeight();
	}

	/**
	 * Renders an element.
	 *
	 * @param g
	 *            the graphics context to render on.
	 * @param element
	 *            the element to render.
	 */
	static void render(GraphicsContext g, StyledRenderable element) {
		Style style = element.getStyle();
		Rectangle remainingBounds = new Rectangle(0, 0, element.getWidth(), element.getHeight());
		StyleHelper.renderWithoutContent(g, remainingBounds, style);
		element.renderContent(g, style, remainingBounds);
	}

	/**
	 * Lays out an element.
	 *
	 * @param widthHint
	 *            the width available for this widget or {@link MWT#NONE}.
	 * @param heightHint
	 *            the height available for this widget or {@link MWT#NONE}.
	 * @param element
	 *            the element to lay out.
	 * @return the element preferred size.
	 */
	static Rectangle validate(int widthHint, int heightHint, StyledRenderable element) {
		Style style = element.getStyle();
		Rectangle boundsHint = new Rectangle(0, 0, widthHint, heightHint);
		Rectangle contentBounds = StyleHelper.computeContentBounds(boundsHint, style);
		contentBounds = element.validateContent(style, contentBounds);
		StyleHelper.computePreferredSize(boundsHint, contentBounds, style);
		return contentBounds;
	}

	static Style getStyleFromStylesheet(Element element) {
		return StyleHelper.getStylesheet().getStyle(element);
	}

	/**
	 * Sets the class selectors.
	 * <p>
	 * The given string is a space separated list of class selectors.
	 *
	 * @param classSelectors
	 *            the class selectors list to split.
	 * @return the class selectors array.
	 */
	static String[] setClassSelectors(String classSelectors) {
		StringTokenizer tokenizer = new StringTokenizer(classSelectors, SPACE);
		int length = tokenizer.countTokens();
		String[] newArray = new String[length];
		int currentIndex = 0;
		while (tokenizer.hasMoreTokens()) {
			newArray[currentIndex++] = tokenizer.nextToken();
		}
		return newArray;
	}

	/**
	 * Adds a class selector.
	 *
	 * @param classSelectors
	 *            the class selectors array.
	 * @param classSelector
	 *            the class selector to add.
	 * @return the modified class selectors array.
	 * @throws IllegalArgumentException
	 *             if the given string contains one or more space.
	 * @throws NullPointerException
	 *             if the given class selector is <code>null</code>.
	 */
	static String[] addClassSelector(String[] classSelectors, String classSelector) {
		if (classSelector.indexOf(' ') != -1) {
			throw new IllegalArgumentException();
		}
		return ArrayTools.add(classSelectors, classSelector);
	}

}
