/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import ej.annotation.Nullable;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementConstructor;
import ej.giml.annotation.ElementRequiredAttribute;
import ej.microui.display.GraphicsContext;
import ej.mwt.MWT;
import ej.style.Style;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;
import ej.style.image.ImageLoader;
import ej.style.util.StyleHelper;
import ej.widget.StyledWidget;

/**
 * A widget that displays an image.
 *
 * @see ej.microui.display.Image
 * @see ImageLoader
 */
@Element
public class Image extends StyledWidget {

	@Nullable
	private ej.microui.display.Image source;

	/**
	 * Creates an image without source.
	 */
	public Image() {
		setEnabled(false);
	}

	/**
	 * Creates an image with the path to the source to display.
	 * <p>
	 * The image loader service is used to load the image from the given path.
	 *
	 * @param source
	 *            the path to the source to display.
	 * @throws NullPointerException
	 *             if the given source is <code>null</code>.
	 * @see ImageLoader#getImage(String)
	 */
	@ElementConstructor
	public Image(@ElementRequiredAttribute String source) {
		this(StyleHelper.getImage(source));
	}

	/**
	 * Creates an image with the source to display.
	 *
	 * @param source
	 *            the source to display.
	 */
	public Image(@Nullable ej.microui.display.Image source) {
		this();
		this.source = source;
	}

	/**
	 * Gets the displayed source of the image.
	 *
	 * @return the displayed source of the image.
	 */
	@Nullable
	public ej.microui.display.Image getSource() {
		return this.source;
	}

	/**
	 * Sets the source to display for this image.
	 *
	 * @param source
	 *            the path to new source to display.
	 * @throws NullPointerException
	 *             if the given source is <code>null</code>.
	 * @see #setSource(ej.microui.display.Image)
	 * @see ImageLoader#getImage(String)
	 * @see #updateSource(String)
	 */
	public void setSource(String source) {
		setSource(StyleHelper.getImage(source));
	}

	/**
	 * Updates the source to display for this image without asking for repaint.
	 *
	 * @param source
	 *            the path to new source to display.
	 * @throws NullPointerException
	 *             if the given source is <code>null</code>.
	 * @see #setSource(ej.microui.display.Image)
	 * @see ImageLoader#getImage(String)
	 * @since 2.3.0
	 */
	public void updateSource(String source) {
		updateSource(StyleHelper.getImage(source));
	}

	/**
	 * Sets the source to display for this image.
	 *
	 * @param source
	 *            the new source to display. Can be <code>null</code>.
	 * @see #updateSource(ej.microui.display.Image)
	 */
	public void setSource(@Nullable ej.microui.display.Image source) {
		updateSource(source);
		repaint();
	}

	/**
	 * Updates the source to display for this image without asking for repaint.
	 *
	 * @param source
	 *            the new source to display. Can be <code>null</code>.
	 * @since 2.3.0
	 */
	public void updateSource(@Nullable ej.microui.display.Image source) {
		this.source = source;
		invalidate();
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle availableBounds) {
		ej.microui.display.Image source = this.source;
		if (source != null) {
			int alignment = style.getAlignment();
			int x = AlignmentHelper.computeXLeftCorner(source.getWidth(), 0, availableBounds.getWidth(), alignment);
			int y = AlignmentHelper.computeYTopCorner(source.getHeight(), 0, availableBounds.getHeight(), alignment);
			g.setColor(style.getForegroundColor());
			g.drawImage(source, x, y, GraphicsContext.LEFT | GraphicsContext.TOP);
		}
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle availableBounds) {
		Rectangle contentSize = new Rectangle();

		ej.microui.display.Image source = this.source;
		if (source != null) {
			int width = getSize(availableBounds.getWidth(), source.getWidth());
			int height = getSize(availableBounds.getHeight(), source.getHeight());
			// int preferredWidth = this.source.getWidth();
			// int width = availableWidth == MWT.NONE ? preferredWidth : Math.min(availableWidth, preferredWidth);
			// int preferredHeight = this.source.getHeight();
			// int height = availableHeight == MWT.NONE ? preferredHeight : Math.min(availableHeight, preferredHeight);
			contentSize.setSize(width, height);
		}

		return contentSize;
	}

	private int getSize(int available, int preferred) {
		return available == MWT.NONE ? preferred : Math.min(available, preferred);
	}

}
