/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.composed;

import java.util.Iterator;

import ej.giml.annotation.ElementChild;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;
import ej.widget.StyledComposite;

/**
 * A composite with only one child.
 * <p>
 * By default it is sized after its child's dimensions. Shall you wish to make the wrapper bigger than its child (for
 * example for layout purposes), use {@link #setAdjustedToChild(boolean)}.
 * <p>
 * Only the area of its child is reactive to the pointer events. That means that if the wrapper is not adjusted to its
 * child, a zone may not react to pointer events. If the wrapper does not have any child, its whole size react to
 * pointer events.
 *
 * @see #contains(int, int)
 */
public class Wrapper extends StyledComposite {

	private boolean adjustedToChild;

	/**
	 * Creates a wrapper that is adjusted to its child.
	 */
	public Wrapper() {
		this.adjustedToChild = true;
	}

	/**
	 * Sets the only child of this composite.
	 *
	 * @param widget
	 *            the widget to add in this composite.
	 */
	@ElementChild(prefix = "setWidget")
	public void setWidget(Widget widget) {
		removeAllWidgets();
		add(widget);
	}

	@Override
	public boolean contains(int x, int y) {
		if (getWidgetsCount() == 0) {
			return super.contains(x, y);
		}
		int relativeX = x - getX();
		int relativeY = y - getY();
		// browse children recursively
		Iterator<Widget> iterator = iterator();
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			Widget at = widget.getWidgetAt(relativeX, relativeY);
			if (at != null) {
				return true;
			}
		}
		return false;
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		if (getWidgetsCount() == 1) {
			Widget widget = getWidget(0);
			if (this.adjustedToChild) {
				widget.validate(widthHint, heightHint);
			} else {
				widget.validate(MWT.NONE, MWT.NONE);
			}
			if (widthHint == MWT.NONE) {
				widthHint = widget.getPreferredWidth();
			}
			if (heightHint == MWT.NONE) {
				heightHint = widget.getPreferredHeight();
			}
		}

		return new Rectangle(0, 0, widthHint, heightHint);
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		if (getWidgetsCount() == 1) {
			Widget widget = getWidget(0);
			int boundsX = bounds.getX();
			int boundsY = bounds.getY();
			int boundsWidth = bounds.getWidth();
			int boundsHeight = bounds.getHeight();
			if (this.adjustedToChild) {
				widget.setBounds(boundsX, boundsY, boundsWidth, boundsHeight);
			} else {
				int preferredWidth = widget.getPreferredWidth();
				int preferredHeight = widget.getPreferredHeight();
				int alignment = getStyle().getAlignment();
				int x = AlignmentHelper.computeXLeftCorner(preferredWidth, boundsX, boundsWidth, alignment);
				int y = AlignmentHelper.computeYTopCorner(preferredHeight, boundsY, boundsHeight, alignment);
				widget.setBounds(x, y, preferredWidth, preferredHeight);
			}
		}
	}

	/**
	 * Gets whether the wrapper is adjusted to its child.
	 *
	 * @return <code>true</code> if the wrapper is adjusted to its child, <code>false</code> otherwise.
	 */
	public boolean isAdjustedToChild() {
		return this.adjustedToChild;
	}

	/**
	 * Sets whether the wrapper is adjusted to its child, meaning that its dimensions fit its child's dimensions.
	 * <p>
	 * When the wrapper is not adjusted to its child, the child's location depends on the wrapper's style alignment
	 * rules.
	 *
	 * @param value
	 *            <code>true</code> to adjust the wrapper's dimensions to its child's dimensions, <code>false</code>
	 *            otherwise.
	 * @see Style#getAlignment()
	 */
	public void setAdjustedToChild(boolean value) {
		this.adjustedToChild = value;
		invalidate();
	}

}
