/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import java.util.Iterator;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementAttribute;
import ej.giml.annotation.ElementChild;
import ej.giml.annotation.ElementConstructor;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.widget.StyledComposite;

/**
 * Lays out its children in a grid.
 * <p>
 * All columns have the same width and all rows have the same height.
 * <p>
 * Horizontal:
 * <p>
 * <img src="doc-files/grid_horizontal.png" alt="Horizontal grid.">
 * <p>
 * Vertical:
 * <p>
 * <img src="doc-files/grid_vertical.png" alt="Vertical grid.">
 */
@Element
public class Grid extends StyledComposite {

	private boolean horizontal;
	private int count;

	/**
	 * Creates a horizontal grid composite.
	 */
	public Grid() {
		this.horizontal = true;
		this.count = 1;
	}

	/**
	 * Creates a grid composite specifying its orientation and number of widgets per line/column (depending on the
	 * orientation chosen via the horizontal parameter).
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 * @param count
	 *            the number of widgets to set.
	 */
	@ElementConstructor
	public Grid(@ElementAttribute(defaultValue = "true") boolean horizontal,
			@ElementAttribute(defaultValue = "1") int count) {
		this.horizontal = horizontal;
		this.count = count;
	}

	/**
	 * Sets the composite orientation: horizontal or vertical.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 */
	public void setHorizontal(boolean horizontal) {
		this.horizontal = horizontal;
		invalidate();
	}

	/**
	 * Sets the number of widgets to set on a line or a column (for horizontal or vertical orientations respectively).
	 *
	 * @param count
	 *            the number of widgets to set.
	 */
	public void setCount(int count) {
		this.count = count;
		invalidate();
	}

	@Override
	@ElementChild(cardinality = Integer.MAX_VALUE, ordered = true)
	public void add(Widget widget) throws NullPointerException, IllegalArgumentException {
		super.add(widget);
	}

	@Override
	public void remove(Widget widget) throws NullPointerException {
		super.remove(widget);
	}

	@Override
	public void removeAllWidgets() {
		super.removeAllWidgets();
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		int length = getWidgetsCount();
		if (length != 0) {
			// Compute widgets expected size.
			boolean horizontalLocal = this.horizontal;
			int countLocal = this.count;
			int otherCount = getOtherCount(length, countLocal);
			int columns;
			int rows;
			if (horizontalLocal) {
				columns = countLocal;
				rows = otherCount;
			} else {
				rows = countLocal;
				columns = otherCount;
			}

			boolean computeWidth = widthHint == MWT.NONE;
			boolean computeHeight = heightHint == MWT.NONE;

			int cellsWidth = computeWidth ? MWT.NONE : widthHint / columns;
			int cellsHeight = computeHeight ? MWT.NONE : heightHint / rows;
			int maxCellWidth = 0;
			int maxCellHeight = 0;

			Iterator<Widget> iterator = iterator();
			while (iterator.hasNext()) {
				Widget widget = iterator.next();
				widget.validate(cellsWidth, cellsHeight);

				// Compute biggest size.
				maxCellWidth = Math.max(maxCellWidth, widget.getPreferredWidth());
				maxCellHeight = Math.max(maxCellHeight, widget.getPreferredHeight());
			}

			// Compute composite preferred size if necessary.
			if (computeWidth) {
				widthHint = maxCellWidth * columns;
			}
			if (computeHeight) {
				heightHint = maxCellHeight * rows;
			}
		}

		// Set composite preferred size.
		return new Rectangle(0, 0, widthHint, heightHint);
	}

	private int getOtherCount(int length, int count) {
		// return (int) Math.ceil((float) length / countLocal);
		return (length + count - 1) / count;
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		int x = bounds.getX();
		int y = bounds.getY();
		int width = bounds.getWidth();
		int height = bounds.getHeight();

		// TODO Try to merge with validate.
		int length = getWidgetsCount();
		if (length == 0) {
			return;
		}

		// Compute widgets bounds.
		boolean horizontalLocal = this.horizontal;
		int countLocal = this.count;
		int otherCount = getOtherCount(length, countLocal);
		int columns;
		int rows;
		if (horizontalLocal) {
			columns = countLocal;
			rows = otherCount;
		} else {
			rows = countLocal;
			columns = otherCount;
		}
		int cellWidth = width / columns;
		int cellHeight = height / rows;

		// Set widgets bounds.
		int offset = 0;
		int currentX = x;
		int currentY = y;
		Iterator<Widget> iterator = iterator();
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			widget.setBounds(currentX, currentY, cellWidth, cellHeight);

			if (++offset == countLocal) {
				// End of line / column.
				offset = 0;
				if (horizontalLocal) {
					currentX = x;
					currentY += cellHeight;
				} else {
					currentY = y;
					currentX += cellWidth;
				}
			} else {
				if (horizontalLocal) {
					currentX += cellWidth;
				} else {
					currentY += cellHeight;
				}
			}
		}
	}

}
