/*
 * Java
 *
 * Copyright 2015-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 */
package ej.util.filemanager.fs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;

import ej.bon.Immortals;
import ej.util.filemanager.FilesManager;

public class LocalFilesManager implements FilesManager {
	private static final String PREFIX = "store";
	private static final boolean DEBUG = true;
	private static final int EXCHANGES_BUFFER_SIZE = 2048;
	private static final Logger LOGGER = Logger.getLogger(LocalFilesManager.class.getName());
	private static final String[] EMPTY = new String[0];
	private final File root;
	private final File tmpDir;
	private final char fileSeparator;
	private final byte[] bytes;

	/**
	 *
	 */
	public LocalFilesManager() {
		this.root = new File(System.getProperty("storage.files.root", "storage"));
		this.tmpDir = new File(this.root, "tmp");
		this.fileSeparator = getFileSeparator();
		this.bytes = createExchangeBuffer();
	}

	@Override
	public void initialize() throws IOException {
		if (DEBUG) {
			LOGGER.finer("LocalFilesManager.initialize(): start");
		}
		if (this.root.exists() && !this.root.isDirectory()) {
			throw new IOException("The specified root directory '" + this.root + "' is not a directory.");
		}

		if (!this.root.exists()) {
			if (!this.root.mkdirs()) {
				throw new IOException("Cannot create the root directory");
			}
		}

		if (this.tmpDir.exists() && !this.tmpDir.isDirectory()) {
			throw new IOException("The specified root directory '" + this.tmpDir + "' is not a directory.");
		}

		if (!this.tmpDir.exists()) {
			if (!this.tmpDir.mkdirs()) {
				throw new IOException("Cannot create the tmp directory");
			}
		}
		if (DEBUG) {
			LOGGER.finer("LocalFilesManager.initialize(): end");
		}
	}

	protected byte[] createExchangeBuffer() {
		return (byte[]) Immortals.setImmortal(new byte[EXCHANGES_BUFFER_SIZE]);
	}

	@Override
	public void shutdown() throws IOException {
		// nothing to do
	}

	@Override
	public synchronized void write(String dirPath, String name, InputStream contents) throws IOException {
		if (DEBUG) {
			LOGGER.finer("Attempting to write " + dirPath + File.pathSeparator + name);
		}
		// Create all parent directories
		File parentDirectory = getParentDirectory(dirPath);
		if (!parentDirectory.exists()) {
			if (!parentDirectory.mkdirs()) {
				throw new IOException("Cannot create parent directories");
			}
		}

		// Create a tmp file.
		File tmpFile = File.createTempFile(PREFIX, name, this.tmpDir);

		// write the contents into that tmp file
		try (FileOutputStream fos = new FileOutputStream(tmpFile)) {
			int r = contents.read(this.bytes);
			while (r != -1) {
				fos.write(this.bytes, 0, r);
				r = contents.read(this.bytes);
			}
		} catch (IOException e) {
			tmpFile.delete();
			throw e;
		}

		// We always override the destination file, therefore delete it if it exist.
		File file = new File(parentDirectory, name);
		if (file.exists()) {
			if (!file.delete()) {
				throw new IOException("Unable to write '" + name + "'");
			}
		}

		// Try to rename the tmp file into the destination file.
		if (!tmpFile.renameTo(file)) {
			throw new IOException("Unable to write '" + name + "'");
		}
		if (DEBUG) {
			LOGGER.finer(file + " has been written.");
		}
	}

	@Override
	public void move(String dirPath, String src, String dst) throws IOException {
		if (DEBUG) {
			LOGGER.finer("Attempting to move " + dirPath + File.pathSeparator + src + " to " + dirPath
					+ File.pathSeparator + dst);
		}
		File srcFile = new File(getParentDirectory(dirPath), src);
		if (!srcFile.exists()) {
			throw new IllegalArgumentException(src + " does not exist.");
		}

		File dstFile = new File(getParentDirectory(dirPath), dst);

		if (!srcFile.renameTo(dstFile)) {
			throw new IOException("Cannot move '" + src + "' to '" + dst + "'.");
		}
		if (DEBUG) {
			LOGGER.finer(srcFile + " moved to " + dstFile);
		}
	}

	@Override
	public boolean exists(String dirPath, String name) throws IOException {
		File file = new File(getParentDirectory(dirPath), name);
		boolean ret = file.exists();
		if (DEBUG) {
			LOGGER.finer("Does " + file + " exist? " + ret);
		}
		return ret;
	}

	@Override
	public InputStream read(String dirPath, String name) {
		try {
			final File file = new File(getParentDirectory(dirPath), name);
			if (DEBUG) {
				LOGGER.finer("Opening " + dirPath + File.pathSeparator + name);
			}
			return new FileInputStream(file);
		} catch (FileNotFoundException e) {
			return null;
		}
	}

	@Override
	public boolean delete(String dirPath, String name) {
		if (DEBUG) {
			LOGGER.finer("Attempting to remove " + dirPath + File.pathSeparator + name);
		}
		File file = new File(getParentDirectory(dirPath), name);
		if (!file.exists()) {
			if (DEBUG) {
				LOGGER.finer(file + " cannot be deleted.");
			}
			throw new IllegalArgumentException(file + " does not exist.");
		}
		boolean ret = file.delete();
		if (DEBUG) {
			LOGGER.finer(file + " removed? " + ret);
		}
		return ret;
	}

	@Override
	public String[] list(String dirPath) throws IOException {
		if (DEBUG) {
			LOGGER.finer("Listing file of" + dirPath);
		}
		String[] list = getParentDirectory(dirPath).list();
		if (list == null) {
			list = EMPTY;
		}
		return list;
	}

	@Override
	public synchronized void clear() throws IOException {
		if (this.root.exists() && this.root.isDirectory()) {
			clear(this.root);
		}
		initialize();
	}

	private void clear(File dir) throws IOException {
		for (File file : dir.listFiles()) {
			if (file.isDirectory()) {
				clear(file);
			}

			if (!file.delete()) {
				throw new IOException("Cannot delete " + file);
			}
		}
	}

	private File getParentDirectory(String dirPath) {
		return new File(this.root, dirPath.replace(File.pathSeparatorChar, this.fileSeparator));
	}

	protected char getFileSeparator() {
		return File.separatorChar;
	}
}
